"""
///
/// b_index.py
///
/// Author:
///   Woo-Kyoung Noh (rainlood@kldp.org)
///
/// (C) 2001 Woo-Kyoung Noh  http://gnome.or.kr/~rainlood/
///
///
///
///
/// <summary>
/// boinda index
/// </summary>
"""

import string,os
from b_meta import BoindaMeta
from b_item import BoindaItem
from b_template import BoindaTemplate
from b_item import is_this_image, is_file_exist, new_thumbnail, is_http, http_thumbnail, is_dir_exist
from b_xml import BoindaXml
from b_setup import BOINDA_ONTEST, BOINDA_ENABLE_TEMPLATES


class BoindaEnv:
	def __init__(self, sSystemRoot='', sWebRoot='', sPath='', dQuery={}):
		self.sSystemRoot=sSystemRoot
		self.sWebRoot=sWebRoot
		self.sPath = sPath
		self.dQuery= dQuery
	def __call__(self, name):
		if self.dQuery.has_key(name):
			return self.dQuery[name]
		else :
			return ''


class BoindaIndex:
	def __init__(self, sSystemRoot='', sWebRoot='', sPath='', dQuery={}):
		env = BoindaEnv(sSystemRoot, sWebRoot, sPath, dQuery)
		paths = ['/']+filter (lambda x:x,  string.split (env.sPath, '/') )
		lenofpath = len(paths)
		bXml = BoindaXml()
		sTemplate = ''
		lLocationPath= []
		for i in range(1, lenofpath):
			indexUrl = os.path.join(env.sSystemRoot, string.join(paths[1:i-1],'/') or '', 'index.xml')
			name = paths[i-1]
			try:
				lItem, tempTemplate = bXml.getIndexAndTemplate(url=indexUrl, urlby=name)
			except:
				lItem=[{'url':name}]
				tempTemplate=''
			if tempTemplate:
				sTemplate=tempTemplate
			dPath={}
			dPath['title'] = name
			if lItem and lItem[0].has_key('title'):
					dPath['title'] = lItem[0]['title']
			dPath['url'] = os.path.join(env.sWebRoot,string.join(paths[1:i], '/'))
			lLocationPath.append(dPath)

		lContentPath = []
		indexUrl = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '', 'index.xml')
		try:
			lContentPath, tempTemplate = bXml.getIndexAndTemplate(url=indexUrl)
		except IOError:
			dirPath = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '')
			if is_dir_exist(dirPath):
				for filename in os.listdir(dirPath):
					lContentPath.append({'url':filename})
		except Exception, e: #IOError
			lContentPath=[{'desc':str(e)}]

		bItemQuery = BoindaItem(dQuery)
		newItem={}
		for dItem in lContentPath:
			item = BoindaItem(dItem)
			if is_http(item('url')):
				fileUrl = item('url')
				if is_this_image(fileUrl):
					thumbnailDirectoryUrl = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '', '.thumbnails')
					thumbnailUrl = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '', '.thumbnails', http_thumbnail(item('url')))
					new_thumbnail(fileUrl, thumbnailDirectoryUrl, thumbnailUrl)
			else:
				fileUrl = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '', item('url'))
				thumbnailDirectoryUrl = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '', '.thumbnails')
				thumbnailUrl = os.path.join(env.sSystemRoot, string.join(paths[1:lenofpath-1],'/') or '', '.thumbnails', item('url'))
				if is_this_image(fileUrl) and not is_file_exist(thumbnailUrl):
					new_thumbnail(fileUrl, thumbnailDirectoryUrl, thumbnailUrl)

			#if bItemQuery('url') and item('url')==bItemQuery('url'):
			#	newItem = dItem
			#	lLocationPath.append(newItem)
			#	break

		#if bItemQuery('url'):
		#	if not newItem:
		#		newItem = {'url':bItemQuery('url')}
		#	lLocationPath.append(newItem)
		#	urlItem = {'title':'DATA', 'desc':bItemQuery('url', q='get urlread')}
		#	lContentPath=[newItem,urlItem]

		self.env = env
		self.env.sDirectoryPath=os.path.join(env.sSystemRoot,string.join(paths[1:lenofpath-1],'/') or '')
		self.lLocationPath = lLocationPath
		self.lContentPath = lContentPath
		self.sTemplate = sTemplate

	def more(self):
		pass

	def getResponse(self):
		if BOINDA_ONTEST:
			import b_template
			reload(b_template)
			BoindaTemplate = b_template.BoindaTemplate
		template = BoindaTemplate(self.env, BoindaMeta(self.lLocationPath), BoindaMeta(self.lContentPath), self.sTemplate)
		return template.getResult()


if __name__ == '__main__':
	from b_test import unittest, BoindaTest
	from b_setup import BOINDA_SYSTEMROOT, BOINDA_MEDUSA_WEBROOT
	sSystemRoot=BOINDA_SYSTEMROOT
	sWebRoot=BOINDA_MEDUSA_WEBROOT
	class BoindaIndexTestCase(unittest.TestCase):
		def setUp(self):
			sPath=''
			dQuery={}
			self.bIndex = BoindaIndex(sSystemRoot, sWebRoot, sPath, dQuery)
		def checkEqual(self):
			#print self.bIndex#.BTemplate
			assert 1==1


	BoindaTest ( BoindaIndexTestCase )
