"""
///
/// b_item.py
///
/// Author:
///   Woo-Kyoung Noh (rainlood@kldp.org)
///
/// (C) 2001 Woo-Kyoung Noh  http://gnome.or.kr/~rainlood/
///
///
///
///
/// <summary>
/// boinda item
/// </summary>
"""

from cStringIO import StringIO
import string, re, os
import mimetypes
import Image
import urllib


def http_thumbnail(sUrl):
	return string.join(re.split('[^\w.]+', sUrl), '_')

def is_http(sUrl):
	if sUrl[:7]=='http://':
		return 1
	else:
		return 0



def get_file_type(fileUrl):
	fileType, fileType2 = mimetypes.guess_type(fileUrl)
	if fileType:
		if fileType2:
			fileType=fileType+fileType2
	else:
		fileType=''
	return fileType

def is_this_image(fileUrl):
	if get_file_type(fileUrl)[:5]=='image':
		return 1
	else:
		return 0

def is_file_exist(fileUrl):
	if os.path.isfile(fileUrl):
		return 1
	else:
		return 0

def is_dir_exist(dirUrl):
	if os.path.isdir(dirUrl):
		return 1
	else:
		return 0

def is_exist(url):
	if os.path.isfile(url):
		return 1
	elif os.path.isdir(url):
		return 1
	else:
		return 0


def new_thumbnail(fileUrl, thumbnailDirectoryUrl, thumbnailUrl):
	try:
		if not os.path.isdir(thumbnailDirectoryUrl):
			os.mkdir(thumbnailDirectoryUrl)
		f=StringIO()
		data = urllib.urlopen(fileUrl).read()
		f.write(data)
		f.seek(0)
		im = Image.open(f, 'r')
		size = im.size

		if size>(100,100):
			if cmp(size[0],size[1])==1:
				maxsize = size[0]
				minsize = size[1]
				tResize = (100,int((minsize*100)/maxsize))
			elif cmp(size[0],size[1])==0:
				tResize = (100,100)
			elif cmp(size[0],size[1])==-1:
				maxsize = size[1]
				minsize = size[0]
				tResize = (int((minsize*100)/maxsize),100)
			im=im.resize(tResize)
		im.save( open(thumbnailUrl, 'wb'), 'png')
	except Exception, e:
		print e
		pass

def cook_link(data):
	pUrl ='(?P<url>\w{1,4}://[^ \t\n\r\f\v<>()]*)'
	p=re.compile(pUrl, re.IGNORECASE)
	r=''
	cursor=0
	while 1:
		m=p.search(data, cursor)
		if m:
			r=r+data[cursor:m.start()]
			cursor=m.end()
			if data[m.start()-1] in ('"','='):
				r=r+data[m.start():m.end()]
			else:
				sUrl=m.group('url')
				r=r+'<a href="%s">%s</a>'%(sUrl, sUrl)
		else:
			break
	data=r+data[cursor:]
	p=re.compile('<a href="%s">%s</a>'%(pUrl,'(?P<title>.*?)'), re.IGNORECASE)
	data=p.sub('<a href="%s">%s</a>(link)' % ('\g<url>', '\g<title>'), data)
	pUrl ='(?P<url>\w{1,4}://[^ \t\n\r\f\v<>()]*)'
	p=re.compile('(?P<email>\w{1,12}@[^ \t\n\r\f\v<>()]*)', re.IGNORECASE)
	data=p.sub('<a href="mailto:%s">%s</a>' % ('\g<email>', '\g<email>'), data)
	return data

class BoindaArg:
	def __init__(self, dItem={}):
		self.dItem=dItem

	def __call__(self, name):
		if self.dItem.has_key(name):
			return self.dItem[name]
		else:
			return ''

class BoindaItem:
	def __init__(self, dItem={}):
		self.dItem=dItem

	def __call__(self, name, **dArg):
		if self.dItem.has_key(name):
			data=self.dItem[name]
			arg = BoindaArg(dArg)
			on =arg('on') or ()
			if name=='desc':
				if 'link' in on:
					data=cook_link(data)
				return data
			elif name=='url':
				if not dArg:
					return data
				# need env
				env =arg('env')
				r=''
				title=arg('title') or data

				anchor_start = 0
				if 'a' in on:
					if is_http(data):
						r=r+'<a href="%s">'%data
						anchor_start = 1
					else:
						if 'exist' in on:
							if is_exist(os.path.join(env.sDirectoryPath, data)):
								r=r+'<a href="%s">' % data
								anchor_start = 1
						else:
								r=r+'<a href="%s">' % data
								anchor_start = 1
				img_start = 0
				if arg('icon'):
						r=r+'<img src="%s" border="0" alt="%s"/>'% (arg('icon'), title)
						img_start = 1
				elif 'img' in on:
					if is_this_image(data):
						if is_http(data):
							thumbnailUrl = os.path.join('.thumbnails',http_thumbnail(data))
						else:
							thumbnailUrl = os.path.join('.thumbnails',data)
						if is_file_exist(os.path.join(env.sDirectoryPath,thumbnailUrl)):
							r=r+'<img src="%s" border="0" alt="%s"/>'% (thumbnailUrl,title)
							img_start = 1
						
				if not img_start:
					if 'b' in on:
						r=r+'<b>'
					r=r+title
					if 'b' in on:
						r=r+'</b>'
				if anchor_start:
					r=r+'</a>'
				if is_http(data):
					r=r+'(link)'
				return r
			else:
				return data
		else :
			if name=='url':
				arg = BoindaArg(dArg)
				title=arg('title')
				icon =arg('icon')
				r=''
				if icon:
					r=r+'<img src="%s" border="0" alt="%s"/>'% (icon, title)
				elif title:
					on =arg('on') or ()
					if 'b' in on:
						r=r+'<b>'
					r=r+title
					if 'b' in on:
						r=r+'</b>'
				return r
			return ''


if __name__ == '__main__':
	from b_test import unittest, BoindaTest
	class BoindaItemTestCase(unittest.TestCase):
		def setUp(self):
			self.item = BoindaItem()

		def checkEqual(self):
			assert 1==1

	BoindaTest ( BoindaItemTestCase )
