"""
///
/// b_medusa.py
///
/// Author:
///   Woo-Kyoung Noh (rainlood@kldp.org)
///
/// (C) 2001 Woo-Kyoung Noh  http://gnome.or.kr/~rainlood/
///
///
///
///
/// <summary>
/// boinda medusa
/// </summary>
"""

import string, sys, cgi, re, stat, os
from cStringIO import StringIO

from b_index import BoindaIndex
from b_setup import BOINDA_SYSTEMROOT
from b_setup import BOINDA_SERVER_HOST, BOINDA_SERVER_PORT
from b_setup import BOINDA_ONTEST
from b_setup import BOINDA_MEDUSA_WEBROOT

from b_item import is_this_image, get_file_type


from b_soap import BoindaSoap


try:
	from medusa import producers
	from medusa import filesys
except:
	print '\n'*10
	print """
Please get medusa at http://www.nightmare.com/medusa/
$ wget http://www.nightmare.com/medusa/medusa-20010416.tar.gz
$ tar xvzf medusa-20010416.tar.gz
$ mv medusa-20010416 medusa
"""
	"""
$ echo "#" > /usr/lib/python2.0/site-packages/medusa/__init__.py
"""



class BoindaHandler:
	def handle_request (self, request):
		self.data=StringIO()
		self.request=request
		if self.request.command == 'post':
			self.request.collector=self
			lenofcontent = string.atoi(self.request.get_header('content-length'))
			self.request.channel.set_terminator(lenofcontent)
		else:
			self.continue_request()

	def continue_request(self):
		data = self.data
		request = self.request
		try:
			if BOINDA_ONTEST:
				import b_soap
				reload(b_soap)
				self.bSoap = b_soap.BoindaSoap()
			resp = self.bSoap.parseData(data)
			request.push(resp)
		except:
			sPath, sParams, sQuery, sFragment = request.split_uri()
			if sQuery:
				dQuery={}
				tempQuery = cgi.parse_qs(sQuery[1:])
				for query in tempQuery.keys():
					dQuery[query]=tempQuery[query][-1]
			else:
				dQuery={}

			sPath = string.replace(sPath, '..', '')
			sSystemRoot = BOINDA_SYSTEMROOT
			sWebRoot = BOINDA_MEDUSA_WEBROOT
			fileSystem=filesys.os_filesystem (sSystemRoot)
			if fileSystem.isfile(sPath):
				request['Content-Length'] = fileSystem.stat(sPath)[stat.ST_SIZE]
				if is_this_image(sPath):
					request['Content-Type'] = get_file_type(sPath)
				else:
					request['Content-Type'] = 'application/x-www-form-urlencoded'
				file = producers.file_producer( fileSystem.open(sPath, 'rb') ).file
				while 1: 
					data = file.read(10000)
					if not data: 
						break 
					request.push(data)
			elif fileSystem.isdir(sPath) and not sPath[-1]=='/':
				request['Location']=sPath+'/'
				request.error(301)
				return
			else:
				if fileSystem.isdir(sPath):
					sPath=sPath+'index.xml'
				if BOINDA_ONTEST:
					import b_index
					reload(b_index)
					BoindaIndex = b_index.BoindaIndex
				bIndex = BoindaIndex(sSystemRoot, sWebRoot, sPath, dQuery)
				resp = bIndex.getResponse()
				request.push(resp)
		request.done()

	def found_terminator (self):
		self.request.channel.set_terminator('\r\n\r\n')
		self.request.collector = None
		self.data = self.data.getvalue()
		self.continue_request()

	def collect_incoming_data (self, data):
		self.data.write(data)

	def match (self, request):
		return 1

	def __init__(self):
		self.bSoap = BoindaSoap()



if __name__ == '__main__':
	from b_test import unittest, BoindaTest
	class BoindaMedusaTestCase(unittest.TestCase):
		def setUp(self):
			pass

		def checkEqual(self):
			assert 1==1

	BoindaTest ( BoindaMedusaTestCase )

	if len(sys.argv)==2 and sys.argv[1]=='server':
		from medusa import http_server
		# http://zope.org/
		REQUEST = re.compile ('([^ ]+) (?P<uri>.*)(( HTTP/([0-9.]+)))')
		#REQUEST = re.compile ('([^ ]+) (?:[^ :?#]+://[^ ?#/]*)?([^ ]+)(( HTTP/([0-9.]+))$|$)')
		def crack_request (r):
			m = REQUEST.match(r)
			if m is not None:
				return string.lower (m.group (1)), m.group('uri'), m.group(5)
			else:
				return None, None, None

		http_server.REQUEST=REQUEST
		http_server.crack_request=crack_request

		from medusa import asyncore
		ip=BOINDA_SERVER_HOST
		port=BOINDA_SERVER_PORT
		server = http_server.http_server(ip, port)

		bHandler=BoindaHandler()
		server.install_handler( bHandler )
		asyncore.loop()
