"""
///
/// b_xml.py
///
/// Author:
///   Woo-Kyoung Noh (rainlood@kldp.org)
///
/// (C) 2001 Woo-Kyoung Noh  http://gnome.or.kr/~rainlood/
///
///
///
///
/// <summary>
/// boinda xml
/// </summary>
"""

import re, urllib
from xml.dom.ext.reader.Sax2 import Reader
from xml.dom import Node

from xml.sax._exceptions import SAXParseException
#from xml.dom.Document import Document
#from xml.xslt import Processor
#from xml.dom.ext import PrettyPrint
#from urllib import urlopen

class BoindaXml:
	def __init__(self):
		self.reader = Reader()

	def getIndexAndTemplate(self, url='', xmlString='', urlby=''):
		dAttr={}
		if urlby:
			dAttr['url'] = urlby
		if url:
			xmlString=urllib.urlopen(url).read()
		return self.getIndexFromString( xmlString, dAttr)

	def getIndexFromString (self, xmlString, dAttr={}):
		self.doc = self.reader.fromString(self.xmlStringToUnicode(xmlString) )
		return self.getIndex(dAttr)

	def xmlStringToUnicode(self, xmlString):
		p=re.compile(' encoding="(?P<e>.*)".*?>', re.I)
		m=p.search(xmlString)
		if m:
			encoding = m.group('e')
			r=unicode(xmlString[m.end():], encoding)
		else:
			encoding = 'utf-8'
			"""
			/// environ['ACCEPT_LANGUAGE']
			"""
			p=re.compile('<\?xml version="1.0">', re.I)
			r=u''+p.sub('', xmlString)
		self.encoding = encoding
		return r

	def dec (self, msg):
		return msg.encode(self.encoding)

	def getIndex (self, dAttr):
		indexElement = self.getIndexElement()
		lItem=[]
		if indexElement:
			lItem=self.getItems (indexElement, dAttr)
			template = indexElement.getAttributeNS('','template')
			return lItem, template
		else:
			return ([],'')

	def getItem (self, itemElement):
		dItem = {}
		for attr in itemElement._get_attributes():
			dItem[self.dec(attr._get_nodeName())] = self.dec(attr._get_nodeValue())
		nodes = filter(lambda x: x.nodeType == Node.ELEMENT_NODE, itemElement.childNodes)
		for node in nodes:
			if node.childNodes:
				dItem[self.dec(node.nodeName)]= self.dec(node.childNodes[0].data)
		return dItem

	def getItems (self, indexElement, dAttr):
		itemElements = self.getItemElements(indexElement)
		lItem=[]
		for itemElement in itemElements:
			if dAttr:
				for attrName in dAttr.keys():
					if self.dec (itemElement.getAttributeNS('', attrName))==dAttr[attrName]:
						lItem.append( self.getItem(itemElement) )

			else:
				lItem.append( self.getItem(itemElement) )
		return lItem

	def getIndexElement (self):
		elements = self.doc.getElementsByTagName('index')
		if elements:
			return elements[0]
		else:
			return ''

	def getItemElements (self, indexElement):
		itemElements = indexElement.getElementsByTagName('item')
		return itemElements


if __name__ == '__main__':
	from b_test import unittest, BoindaTest
	class BoindaXmlTestCase(unittest.TestCase):
		def setUp(self):
			self.bXml = BoindaXml()

		def checkFromXmlString(self):
			lItem, sTemplate = self.bXml.getIndexAndTemplate(xmlString="""\
<?xml version="1.0"?>

<index template="gnome or kr template">

<item url="whoami">
  <title>profile</title>
</item>

<item url="http://gnome.or.kr" type="html">
  <title>Korean GNOME Users</title>
</item>
</index>
""")
			assert lItem==[{'title': 'profile', 'url': 'whoami'}, {'title': 'Korean GNOME Users', 'type': 'html', 'url': 'http://gnome.or.kr'}]
			assert sTemplate=='gnome or kr template'

		def checkFromUrl(self):
			lItem, sTemplate = self.bXml.getIndexAndTemplate(url='index.xml')
			assert sTemplate==''

	BoindaTest ( BoindaXmlTestCase )
